@@ -2,6 +2,7 @@ from collections import defaultdict
 from functools import update_wrapper
 
 from django import forms
+from django.contrib import messages
 from django.contrib.admin import ModelAdmin, SimpleListFilter, helpers
 from django.contrib.admin.options import IncorrectLookupParameters
 from django.contrib.admin.utils import unquote
@@ -13,7 +14,6 @@ from django.utils.html import format_html, mark_safe
 from django.utils.text import capfirst
 from django.utils.translation import gettext_lazy as _
 from django.views.decorators.csrf import csrf_protect
-
 from tree_queries.forms import TreeNodeChoiceField
 
 
@@ -105,7 +105,7 @@ class TreeAdmin(ModelAdmin):
         return format_html(
             '<a href="{}">{}</a>',
             reverse(
-                "admin:%s_%s_move" % (opts.app_label, opts.model_name),
+                f"admin:{opts.app_label}_{opts.model_name}_move",
                 args=(instance.pk,),
             ),
             _("move"),
@@ -206,12 +206,20 @@ class MoveForm(forms.Form):
 
         super().__init__(*args, **kwargs)
 
-        queryset = self.model._default_manager.with_tree_fields()
-        self.fields["move_to"] = forms.ChoiceField(
-            label=capfirst(_("move to")),
+        choices = self._generate_choices(self.model._default_manager.with_tree_fields())
+        self.fields["new_location"] = forms.ChoiceField(
+            label=_("New location"),
             widget=forms.RadioSelect,
-            choices=self._generate_choices(queryset),
+            choices=choices,
         )
+        if len(choices) <= 1:
+            messages.warning(
+                self.request,
+                _(
+                    "Moving isn't possible because there are no valid targets."
+                    " Maybe you selected the only root node?"
+                ),
+            )
 
     def _generate_choices(self, queryset):
         children = defaultdict(list)
@@ -219,14 +227,14 @@ class MoveForm(forms.Form):
             children[node.parent_id].append(node)
 
         def _text_indent(depth):
-            return mark_safe(' style="text-indent:{}px"'.format(depth * 30))
+            return mark_safe(f' style="text-indent:{depth * 30}px"')
 
         choices = []
 
         def _iterate(parent_id):
-            for node in children[parent_id]:
+            for index, node in enumerate(children[parent_id]):
                 if node == self.instance:
-                    choices[-1] = (
+                    choice = (
                         "",
                         format_html(
                             '<div class="mv is-self"{}><strong>{}</strong>',
@@ -234,11 +242,20 @@ class MoveForm(forms.Form):
                             node,
                         ),
                     )
+                    if index == 0 and parent_id:
+                        # Moving the first child of parent_id; do not remove parent_id
+                        choices[-1] = (
+                            choices[-1][0],
+                            mark_safe(choices[-1][1].replace("mv-mark", "hidden")),
+                        )
+                        choices.append(choice)
+                    else:
+                        choices[-1] = choice
                     continue
 
                 choices.append(
                     (
-                        "{}:first".format(node.id),
+                        f"{node.id}:first",
                         format_html(
                             '<div class="mv to-first"{}><strong>{}</strong>'
                             '<div class="mv-mark"{}>&rarr; {}</div></div>',
@@ -252,7 +269,7 @@ class MoveForm(forms.Form):
                 _iterate(node.id)
                 choices.append(
                     (
-                        "{}:right".format(node.id),
+                        f"{node.id}:right",
                         format_html(
                             '<div class="mv to-right mv-mark"{}>&rarr; {}</div>',
                             _text_indent(node.tree_depth),
@@ -275,10 +292,10 @@ class MoveForm(forms.Form):
 
     def clean(self):
         data = super().clean()
-        if not data.get("move_to"):
+        if not data.get("new_location"):
             return data
 
-        pk, _sep, first_or_right = data["move_to"].partition(":")
+        pk, _sep, first_or_right = data["new_location"].partition(":")
         data["first_or_right"] = first_or_right
 
         if pk == "0":
@@ -329,7 +346,7 @@ class MoveForm(forms.Form):
         )
 
         opts = self.modeladmin.model._meta
-        return redirect("admin:%s_%s_changelist" % (opts.app_label, opts.model_name))
+        return redirect(f"admin:{opts.app_label}_{opts.model_name}_changelist")
 
 
 class CloneForm(forms.Form):
@@ -369,9 +386,9 @@ class CloneForm(forms.Form):
             if field.auto_created or not field.editable:
                 continue
 
-            self.fields["set_{}".format(field.name)] = forms.BooleanField(
+            self.fields[f"set_{field.name}"] = forms.BooleanField(
                 label=(
-                    "{} ({})".format(capfirst(field.verbose_name), field.name)
+                    f"{capfirst(field.verbose_name)} ({field.name})"
                     if hasattr(field, "verbose_name")
                     else field.name
                 ),
@@ -389,7 +406,7 @@ class CloneForm(forms.Form):
             raise forms.ValidationError({"target": _("Cannot clone node to itself.")})
 
         for field in self.instance._meta.get_fields():
-            if self.cleaned_data.get("set_{}".format(field.name)):
+            if self.cleaned_data.get(f"set_{field.name}"):
                 setattr(target, field.name, getattr(self.instance, field.name))
 
         # All fields of model are not in this form
@@ -402,7 +419,7 @@ class CloneForm(forms.Form):
         fields = []
 
         for field in self.instance._meta.get_fields():
-            if self.cleaned_data.get("set_{}".format(field.name)):
+            if self.cleaned_data.get(f"set_{field.name}"):
                 setattr(target, field.name, getattr(self.instance, field.name))
                 fields.append("{}".format(getattr(field, "verbose_name", field.name)))
 
@@ -440,9 +457,7 @@ class CloneForm(forms.Form):
         target.save()
 
         opts = self.modeladmin.model._meta
-        return redirect(
-            "admin:%s_%s_change" % (opts.app_label, opts.model_name), target.pk
-        )
+        return redirect(f"admin:{opts.app_label}_{opts.model_name}_change", target.pk)
 
 
 class AncestorFilter(SimpleListFilter):
@@ -462,7 +477,7 @@ class AncestorFilter(SimpleListFilter):
         admin.site.register(Node, NodeAdmin)
     """
 
-    title = _("ancestor")
+    title = _("subtree")
     parameter_name = "ancestor"
     max_depth = 1
 
