@@ -2,10 +2,9 @@ from django.conf import settings
 from django.db import models
 from django.db.models import Q, signals
 from django.utils.translation import activate, get_language, gettext_lazy as _
-
 from tree_queries.fields import TreeNodeForeignKey
 
-from feincms3.utils import validation_error
+from feincms3.utils import ChoicesCharField, validation_error
 
 
 class MenuMixin(models.Model):
@@ -15,12 +14,7 @@ class MenuMixin(models.Model):
     and a meta navigation (containing contact, imprint etc.)
     """
 
-    menu = models.CharField(
-        _("menu"),
-        max_length=20,
-        blank=True,
-        choices=(("", ""),),  # Non-empty choices for get_*_display
-    )
+    menu = ChoicesCharField(_("menu"), max_length=100, blank=True)
 
     class Meta:
         abstract = True
@@ -48,11 +42,7 @@ class TemplateMixin(models.Model):
     ``Template`` instances through Django's administration interface.
     """
 
-    template_key = models.CharField(
-        _("template"),
-        max_length=100,
-        choices=(("", ""),),  # Non-empty choices for get_*_display
-    )
+    template_key = ChoicesCharField(_("template"), max_length=100)
 
     class Meta:
         abstract = True
@@ -71,7 +61,7 @@ class TemplateMixin(models.Model):
         Return the selected template instances' ``regions`` attribute, falling
         back to an empty list if no template instance could be found.
         """
-        return self.template.regions if self.template else []
+        return self.template.regions
 
     @staticmethod
     def fill_template_key_choices(sender, **kwargs):
@@ -94,7 +84,7 @@ class LanguageMixin(models.Model):
     Pages may come in varying languages. ``LanguageMixin`` helps with that.
     """
 
-    language_code = models.CharField(
+    language_code = ChoicesCharField(
         _("language"),
         max_length=10,
         choices=settings.LANGUAGES,
@@ -192,7 +182,9 @@ class RedirectMixin(models.Model):
     The ``RedirectMixin`` allows adding redirects in the page tree.
     """
 
-    redirect_to_url = models.CharField(_("Redirect to URL"), max_length=200, blank=True)
+    redirect_to_url = models.CharField(
+        _("Redirect to URL"), max_length=1000, blank=True
+    )
     redirect_to_page = TreeNodeForeignKey(
         "self",
         on_delete=models.SET_NULL,
@@ -205,6 +197,15 @@ class RedirectMixin(models.Model):
     class Meta:
         abstract = True
 
+    def get_redirect_url(self):
+        """
+        Return the URL for the redirect, if a redirect is configured
+        """
+        if self.redirect_to_url:
+            return self.redirect_to_url
+        elif self.redirect_to_page:
+            return self.redirect_to_page.get_absolute_url()
+
     def clean_fields(self, exclude=None):
         """
         Ensure that redirects are configured properly.
@@ -227,8 +228,8 @@ class RedirectMixin(models.Model):
                 )
 
             if (
-                self.redirect_to_page.redirect_to_page_id
-                or self.redirect_to_page.redirect_to_url
+                self.redirect_to_page.redirect_to_url
+                or self.redirect_to_page.redirect_to_page
             ):
                 raise validation_error(
                     _(
@@ -237,11 +238,7 @@ class RedirectMixin(models.Model):
                     )
                     % {
                         "title": self.redirect_to_page,
-                        "path": (
-                            self.redirect_to_page.redirect_to_page.get_absolute_url()
-                            if self.redirect_to_page.redirect_to_page
-                            else self.redirect_to_page.redirect_to_url
-                        ),
+                        "path": self.redirect_to_page.get_redirect_url(),
                     },
                     field="redirect_to_page",
                     exclude=exclude,
@@ -258,8 +255,7 @@ class RedirectMixin(models.Model):
                     )
                     % {
                         "pages": ", ".join(
-                            "%s (%s)" % (page, page.get_absolute_url())
-                            for page in other
+                            f"{page} ({page.get_absolute_url()})" for page in other
                         )
                     },
                     field="redirect_to_page",
